from rocolib.api.components.Component import Component

c = Component()

# IMPORTANT:There is no specific microcontroller for this car. To use this, create a specific car builder
# and adjust the constraints of this component using your preferred microcontroller.
# Default servo: FS90R servo see dimensions in dimensions.py

c.addParameter("driveservo", "fs90r", paramType="dimension")

#Parameters for the sheath that would cover the wheels and brains of the car (changing these will change dimension of a car)
c.addParameter("length", 80, paramType="length", minValue=65) #tested minValue
c.addParameter("height", 36, paramType="length", minValue=13) #tested minValue
c.addParameter("width", 60, paramType="length", minValue=60) #tested minValue

#Cover/body of the car (Separate rectangles because the edges are customized)
for i in range(1, 3):
    c.addSubcomponent("rect%d" %i, "Rectangle")
    c.addConstraint(("rect%d" %i, "l"), "length")
    c.addConstraint(("rect%d" %i, "w"), "height")

for i in range(3, 5):
    c.addSubcomponent("rect%d" %i, "Rectangle")
    c.addConstraint(("rect%d" %i, "l"), "length")
    c.addConstraint(("rect%d" %i, "w"), "width")

c.addConnection(("rect3", "b"), ("rect1", "b"), angle=-90)
c.addConnection(("rect2", "b"), ("rect4", "b"), angle=-90)
c.addConnection(("rect4", "t"), ("rect1", "t"), angle=-90)

#Attaching servos to the car
c.addSubcomponent("rect0", "Rectangle")
c.addConstraint(("rect0", "l"), ("height", "driveservo"), "x[0]-getDim(x[1], 'motorwidth')")
c.addConstraint(("rect0", "w"), "driveservo", "getDim(x, 'motorwidth')")

for i in range(2):
    c.addSubcomponent("sheathsplit%d" %i, "SplitEdge")
    c.addConstraint(("sheathsplit%d" %i, "toplength"), "length", "(x,)")


c.addConstraint(("sheathsplit0", "botlength"),
                ("length", "driveservo"),
                "(x[0]/2. - getDim(x[1], 'motorwidth')  - 2 * getDim(x[1], 'hornoffset'), "
                "getDim(x[1], 'motorwidth'), x[0]/2. + 2 * getDim(x[1], 'hornoffset'))")

c.addConstraint(("sheathsplit1", "botlength"),
                ("length", "driveservo"),
                "(x[0]/2. + 2 * getDim(x[1], 'hornoffset'), "
                "getDim(x[1], 'motorwidth'), x[0]/2. - getDim(x[1], 'motorwidth')  - 2 * getDim(x[1], 'hornoffset'))")

c.addConnection(("sheathsplit0", "topedge0"), ("rect2", "t"))
c.addConnection(("sheathsplit0", "botedge1"), ("rect0", "r"), angle=180)

for i in range(2):
    c.addSubcomponent("rTriangle%d" %i, "Right45Triangle")
    c.addConstraint(("rTriangle%d" %i, "base"), "driveservo", "getDim(x, 'motorwidth')")
    c.addConstraint(("rTriangle%d" %i, "height"), "driveservo", "getDim(x, 'motorwidth')")

c.addConnection(("rect0", "l"), ("rTriangle0", "bedge"))
c.addConnection(("rTriangle0", "hedge"), ("rTriangle1", "hedge"), angle=180)

#Adding tabs
c.addConnection(("sheathsplit1", "topedge0"), ("rect3", "t"))
c.addConnection(("sheathsplit1", "botedge0"), ("sheathsplit0", "botedge2"), angle=-180, tabWidth=7)

#Four wheels using Wheel.yaml
c.addSubcomponent("fleft", "Wheel", invert=True, inherit="angle")
c.addSubcomponent("bright", "Wheel", invert=True, inherit="angle")

#Setting the parameters for the Wheels
for servo in ("bright", "fleft"):
    c.addConstraint((servo, "servo"), "driveservo")
    c.addConstraint((servo, "length"), ("length", "driveservo"), '((x[0]/2. + 2 * getDim(x[1], "hornoffset")))')
    c.addConstConstraint((servo, "center"), False)
    c.addConstraint((servo, "radius"), "height", '(x / 1.35)')

#Support for wheels so that they won't collapse or wiggle
c.addSubcomponent("between1", "SimpleRectBeam")
c.addConstraint(("between1", "length"), ("width", "driveservo"), "x[0]- 2 * getDim(x[1], 'motorheight')")
c.addConstraint(("between1", "depth"), "driveservo", "getDim(x, 'motorwidth')")
c.addConstraint(("between1", "width"), "driveservo", "getDim(x, 'motorwidth')")

#Back between support
c.addConnection(("fleft", "topedge2"), ("between1", "topedge1"), angle=90)
c.addConnection(("between1", "botedge1"), ("bright", "botedge2"), angle=90)

# c.addConnection(("between2", "topedge1"), ("bright", "topedge2"), angle=90)
# c.addConnection(("between2", "botedge1"), ("fleft", "botedge2"), angle=90, tabWidth=10)

# Flipping right wheels to match the orientation of left wheels
c.addConstConstraint(("bright", "flip"), True)

c.addConnection(("rTriangle1", "bedge"), ("fleft", "botedge0"))

#Inheriting interfaces to make this Component reusable
c.inheritAllInterfaces("fleft")
c.inheritAllInterfaces("bright")
c.inheritAllInterfaces("rect1")
c.inheritAllInterfaces("rect2")
c.inheritAllInterfaces("rect3")
c.inheritAllInterfaces("rect4")

c.toLibrary("HalfCar")