from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face
from rocolib.utils.numsym import sin, cos, tan, deg2rad


class Kite(FoldedComponent):
  def define(self):
    self.addParameter("thickness", 10, paramType="length")
    self.addParameter("angle", 45, paramType="angle")

    self.addEdgeInterface("b", "kite.e0", "thickness")
    self.addEdgeInterface("t", "kite.e3", "thickness")

  def assemble(self):
    t = self.getParameter("thickness")
    a = self.getParameter("angle")
    a2 = deg2rad(a/2.)
    ar = deg2rad(a)

    s = Face("", ((t, 0), (t, t * tan(a2)), (t * cos(ar), t * sin(ar)), (0,0)))

    self.addFace(s, "kite")

if __name__ == "__main__":
    Kite.test()
